/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#include "stdafx.h"

#ifdef _USRDLL
#define _QUBFITS_LIB_
#endif 

#include "QUB_BasicCurve.h"

typedef struct {
	double Base;
} ConstComponent;

static ConstComponent CONST_DEFAULTS = {0.0};

#define CONST_NPAR 1

static char CONST_PARAM_NAMES[][MAX_NAMELEN] = {"Base"};


class QUB_Curve_Const : public QUB_BasicCurve<ConstComponent, CONST_NPAR>
{
public:
	QUB_Curve_Const()
		: QUB_BasicCurve<ConstComponent, CONST_NPAR>(
				"Const",
				"Base",
				&(CONST_PARAM_NAMES[0][0]),
				CONST_DEFAULTS,
				CreateNullCurve() )
	{}
	virtual void evalAll(QF_Data *data, double *yy, int n, double **vvv)
	{
		evalOne(data, yy, 0);
	}
	virtual void evalOne(QF_Data *data, double *yy, int c)
	{
		double Base = components[c].Base;
		int n = data->n;
		for (int i=0; i<n; ++i)
			yy[i] += Base;
	}
};

QUBFITS_API IQUB_Curve* __stdcall CreateConst()
{
	return new QUB_Curve_Const;
}




// QUB_Curve_Linear, new QUB_Curve_Linear

typedef struct {
	double Slope, Icept;
} LinearComponent;

static LinearComponent LIN_DEFAULTS = {1.0, 1.0};

#define LIN_NPAR 2

static char LIN_PARAM_NAMES[][MAX_NAMELEN] = {"Slope", "Icept"};


class QUB_Curve_Linear : public QUB_BasicCurve<LinearComponent, LIN_NPAR>
{
public:
	QUB_Curve_Linear()
		: QUB_BasicCurve<LinearComponent, LIN_NPAR>(
				"Linear",
				"Slope*(x-x0) + Icept",
				&(LIN_PARAM_NAMES[0][0]),
				LIN_DEFAULTS,
				CreateNullCurve() )
	{}
	virtual void evalOne(QF_Data *data, double *yy, int c)
	{
		double x0 = getX0();
		double m = components[c].Slope;
		double b = components[c].Icept;
		int n = data->n;
		double *xx = data->xx;
		for (int i=0; i<n; ++i)
			yy[i] += m * (xx[i]-x0) + b;
	}
};

QUBFITS_API IQUB_Curve* __stdcall CreateLinear()
{
	return new QUB_Curve_Linear;
}



